@@ -14,7 +14,6 @@ from _pygalmesh import (
     RingExtrude,
     Rotate,
     Scale,
-    SizingFieldBase,
     Stretch,
     Tetrahedron,
     Torus,
@@ -22,22 +21,27 @@ from _pygalmesh import (
     Union,
 )
 
-from .__about__ import __version__
+from . import _cli
+from .__about__ import __cgal_version__, __version__
 from .main import (
+    generate_2d,
+    generate_from_array,
     generate_from_inr,
     generate_mesh,
     generate_periodic_mesh,
     generate_surface_mesh,
     generate_volume_mesh_from_surface_mesh,
-    generate_with_sizing_field,
     remesh_surface,
+    save_inr,
 )
 
 __all__ = [
     "__version__",
+    "__cgal_version__",
+    #
+    "_cli",
     #
     "DomainBase",
-    "SizingFieldBase",
     "Translate",
     "Rotate",
     "Scale",
@@ -58,10 +62,12 @@ __all__ = [
     "RingExtrude",
     #
     "generate_mesh",
-    "generate_with_sizing_field",
+    "generate_2d",
     "generate_periodic_mesh",
     "generate_surface_mesh",
     "generate_volume_mesh_from_surface_mesh",
+    "generate_from_array",
     "generate_from_inr",
     "remesh_surface",
+    "save_inr",
 ]
