@@ -1,6 +1,9 @@
+from __future__ import annotations
+
 import math
 import os
 import tempfile
+from typing import Callable
 
 import meshio
 import numpy
@@ -28,20 +31,22 @@ class Wrapper(SizingFieldBase):
 
 def generate_mesh(
     domain,
-    extra_feature_edges=None,
-    bounding_sphere_radius=0.0,
-    lloyd=False,
-    odt=False,
-    perturb=True,
-    exude=True,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    max_circumradius_edge_ratio=0.0,
-    max_cell_circumradius=0.0,
-    verbose=True,
-    seed=0,
+    extra_feature_edges: list | None = None,
+    bounding_sphere_radius: float = 0.0,
+    lloyd: bool = False,
+    odt: bool = False,
+    perturb: bool = True,
+    exude: bool = True,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float | Callable[..., float] = 0.0,
+    max_facet_distance: float = 0.0,
+    max_circumradius_edge_ratio: float = 0.0,
+    max_cell_circumradius: float | Callable[..., float] = 0.0,
+    exude_time_limit: float = 0.0,
+    exude_sliver_bound: float = 0.0,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     """
     From <https://doc.cgal.org/latest/Mesh_3/classCGAL_1_1Mesh__criteria__3.html>:
@@ -118,6 +123,8 @@ def generate_mesh(
         max_circumradius_edge_ratio=max_circumradius_edge_ratio,
         max_cell_circumradius_value=max_cell_circumradius_value,
         max_cell_circumradius_field=max_cell_circumradius_field,
+        exude_time_limit=exude_time_limit,
+        exude_sliver_bound=exude_sliver_bound,
         verbose=verbose,
         seed=seed,
     )
@@ -130,9 +137,9 @@ def generate_mesh(
 def generate_2d(
     points,
     constraints,
-    B=math.sqrt(2),
-    max_edge_size=0.0,
-    num_lloyd_steps=0,
+    B: float = math.sqrt(2),
+    max_edge_size: float = 0.0,
+    num_lloyd_steps: int = 0,
 ):
     # some sanity checks
     points = numpy.asarray(points)
@@ -158,19 +165,19 @@ def generate_2d(
 def generate_periodic_mesh(
     domain,
     bounding_cuboid,
-    lloyd=False,
-    odt=False,
-    perturb=True,
-    exude=True,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    max_circumradius_edge_ratio=0.0,
-    max_cell_circumradius=0.0,
-    number_of_copies_in_output=1,
-    verbose=True,
-    seed=0,
+    lloyd: bool = False,
+    odt: bool = False,
+    perturb: bool = True,
+    exude: bool = True,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_facet_distance: float = 0.0,
+    max_circumradius_edge_ratio: float = 0.0,
+    max_cell_circumradius: float = 0.0,
+    number_of_copies_in_output: int = 1,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     fh, outfile = tempfile.mkstemp(suffix=".mesh")
     os.close(fh)
@@ -203,12 +210,12 @@ def generate_periodic_mesh(
 
 def generate_surface_mesh(
     domain,
-    bounding_sphere_radius=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    verbose=True,
-    seed=0,
+    bounding_sphere_radius: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_facet_distance: float = 0.0,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     fh, outfile = tempfile.mkstemp(suffix=".off")
     os.close(fh)
@@ -230,20 +237,22 @@ def generate_surface_mesh(
 
 
 def generate_volume_mesh_from_surface_mesh(
-    filename,
-    lloyd=False,
-    odt=False,
-    perturb=True,
-    exude=True,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    max_circumradius_edge_ratio=0.0,
-    max_cell_circumradius=0.0,
-    verbose=True,
-    reorient=False,
-    seed=0,
+    filename: str,
+    lloyd: bool = False,
+    odt: bool = False,
+    perturb: bool = True,
+    exude: bool = True,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_facet_distance: float = 0.0,
+    max_circumradius_edge_ratio: float = 0.0,
+    max_cell_circumradius: float = 0.0,
+    exude_time_limit: float = 0.0,
+    exude_sliver_bound: float = 0.0,
+    verbose: bool = True,
+    reorient: bool = False,
+    seed: int = 0,
 ):
     mesh = meshio.read(filename)
 
@@ -267,6 +276,8 @@ def generate_volume_mesh_from_surface_mesh(
         max_facet_distance=max_facet_distance,
         max_circumradius_edge_ratio=max_circumradius_edge_ratio,
         max_cell_circumradius=max_cell_circumradius,
+        exude_time_limit=exude_time_limit,
+        exude_sliver_bound=exude_sliver_bound,
         verbose=verbose,
         reorient=reorient,
         seed=seed,
@@ -279,19 +290,21 @@ def generate_volume_mesh_from_surface_mesh(
 
 
 def generate_from_inr(
-    inr_filename,
-    lloyd=False,
-    odt=False,
-    perturb=True,
-    exude=True,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    max_circumradius_edge_ratio=0.0,
-    max_cell_circumradius=0.0,
-    verbose=True,
-    seed=0,
+    inr_filename: str,
+    lloyd: bool = False,
+    odt: bool = False,
+    perturb: bool = True,
+    exude: bool = True,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_facet_distance: float = 0.0,
+    max_circumradius_edge_ratio: float = 0.0,
+    max_cell_circumradius: float | dict[int | str, float] = 0.0,
+    exude_time_limit: float = 0.0,
+    exude_sliver_bound: float = 0.0,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     fh, outfile = tempfile.mkstemp(suffix=".mesh")
     os.close(fh)
@@ -310,6 +323,8 @@ def generate_from_inr(
             max_facet_distance=max_facet_distance,
             max_circumradius_edge_ratio=max_circumradius_edge_ratio,
             max_cell_circumradius=max_cell_circumradius,
+            exude_time_limit=exude_time_limit,
+            exude_sliver_bound=exude_sliver_bound,
             verbose=verbose,
             seed=seed,
         )
@@ -348,13 +363,13 @@ def generate_from_inr(
 
 
 def remesh_surface(
-    filename,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_facet_distance=0.0,
-    verbose=True,
-    seed=0,
+    filename: str,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_facet_distance: float = 0.0,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     mesh = meshio.read(filename)
 
@@ -382,13 +397,13 @@ def remesh_surface(
     return mesh
 
 
-def save_inr(vol, h, fname):
+def save_inr(vol, voxel_size: tuple[float, float, float], fname: str):
     """
     Save a volume (described as a numpy array) to INR format.
     Code inspired by iso2mesh (http://iso2mesh.sf.net) by Q. Fang
     INPUTS:
-    - vol: volume as numpy array
-    - h: voxel sizes as list or numpy array
+    - vol: volume as a 3D numpy array
+    - h: voxel sizes
     - fname: filename for saving the inr file
     """
     fid = open(fname, "wb")
@@ -400,10 +415,23 @@ def save_inr(vol, h, fname):
         "float64": ("float", 64),
     }[vol.dtype.name]
 
-    header = (
-        "#INRIMAGE-4#{8:s}\nXDIM={0:d}\nYDIM={1:d}\nZDIM={2:d}\nVDIM=1\nTYPE={3:s}\n"
-        + "PIXSIZE={4:d} bits\nCPU=decm\nVX={5:f}\nVY={6:f}\nVZ={7:f}\n"
-    ).format(*vol.shape, btype, bitlen, h[0], h[1], h[2], "{")
+    xdim, ydim, zdim = vol.shape
+    header = "\n".join(
+        [
+            "#INRIMAGE-4#{",
+            f"XDIM={xdim}",
+            f"YDIM={ydim}",
+            f"ZDIM={zdim}",
+            "VDIM=1",
+            f"TYPE={btype}",
+            f"PIXSIZE={bitlen} bits",
+            "CPU=decm",
+            f"VX={voxel_size[0]:f}",
+            f"VY={voxel_size[1]:f}",
+            f"VZ={voxel_size[2]:f}",
+        ]
+    )
+    header += "\n"
 
     header = header + "\n" * (256 - 4 - len(header)) + "##}\n"
 
@@ -413,24 +441,24 @@ def save_inr(vol, h, fname):
 
 def generate_from_array(
     vol,
-    h,
-    lloyd=False,
-    odt=False,
-    perturb=True,
-    exude=True,
-    max_edge_size_at_feature_edges=0.0,
-    min_facet_angle=0.0,
-    max_radius_surface_delaunay_ball=0.0,
-    max_cell_circumradius=0.0,
-    max_facet_distance=0.0,
-    max_circumradius_edge_ratio=0.0,
-    verbose=True,
-    seed=0,
+    voxel_size: tuple[float, float, float],
+    lloyd: bool = False,
+    odt: bool = False,
+    perturb: bool = True,
+    exude: bool = True,
+    max_edge_size_at_feature_edges: float = 0.0,
+    min_facet_angle: float = 0.0,
+    max_radius_surface_delaunay_ball: float = 0.0,
+    max_cell_circumradius: float | dict[int | str, float] = 0.0,
+    max_facet_distance: float = 0.0,
+    max_circumradius_edge_ratio: float = 0.0,
+    verbose: bool = True,
+    seed: int = 0,
 ):
     assert vol.dtype in ["uint8", "uint16"]
     fh, inr_filename = tempfile.mkstemp(suffix=".inr")
     os.close(fh)
-    save_inr(vol, h, inr_filename)
+    save_inr(vol, voxel_size, inr_filename)
     mesh = generate_from_inr(
         inr_filename,
         lloyd,
